## importfile.m - Imports results data, post processes results & calcuates
##                cost function
# Copyright 2009,2010  Richard James MacPherson, Grant Ingram and Jamie McIntosh
# This file is part of PEW.

#    PEW is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    PEW is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with PEW.  If not, see <http://www.gnu.org/licenses/>.

#####################
# Revision History - Jamie McIntosh
# 1.01 - ssh connection now possible from outside of university network
# 1.02 - Survivors of each generation are copied into the next Generation
#        filestore so that repeat testing is removed and script modified
#        to skip these chromosomes in CFD analysis
# 1.03 - Reverted to 1.01 - skipping affects mutation
# 1.04 - cske is now correctly pulled from result.are file (as opposed to
#        vorticity
# 1.05 - Added support for datalink function
# 1.06 - Log file now working as intended
# 1.07 - U^3 integral support added
# 1.08 - Streamline data processing support added
# 1.09 - Non intensive jobs, ie server up/downloading now performed on vega
#
#####################
# Known Issues
# 1. Field and Pitch need to be compiled and put in /usr/bin/ directory.
#    oegularise.m needs to be given ug+x chmod rights and renamed and copied
#    to /usr/bin directory without extension (ie oegularise)
#####################

function importfile()

global iga; global base_dir; global cluster_dir; global job_id;
global popsize; global cost; global npar; global par; global user_id;
global keep; global datalink;

## Process Results for Each Chromosome
for chrom=1:1:(popsize)
  printf("\nChrom: %d",chrom);
  
  sys19 = sprintf("mkdir -p %sGeneration_%d/chr_%d/",base_dir,iga,chrom);
  system(sys19);
  chdir(sprintf("%sGeneration_%d/chr_%d/",base_dir,iga,chrom));

  ## Copy Results Files Across From Server 
  if datalink == 1 % Use database - no space saving
    sys20 = sprintf("scp %s@vega3.dur.ac.uk:%sGeneration_%d/chr_%d/* ./"
                    ,user_id,cluster_dir,iga,chrom);
    system(sys20); # Copy all files files from server
  else
    sys20 = sprintf("scp %s@vega3.dur.ac.uk:%sGeneration_%d/chr_%d/*.out ./"
                    ,user_id,cluster_dir,iga,chrom);
    sys21 = sprintf("scp %s@vega3.dur.ac.uk:%sGeneration_%d/chr_%d/ref ./"
                    ,user_id,cluster_dir,iga,chrom);
    sys22 = sprintf("scp %s@vega3.dur.ac.uk:%sGeneration_%d/chr_%d/res ./"
                    ,user_id,cluster_dir,iga,chrom);
    sys23 = sprintf("scp %s@vega3.dur.ac.uk:%sGeneration_%d/chr_%d/ucubed.srp ./"
                    ,user_id,cluster_dir,iga,chrom);
    sys24 = sprintf("scp %s@vega3.dur.ac.uk:%sGeneration_%d/chr_%d/streamdata ./"
                    ,user_id,cluster_dir,iga,chrom);
    system(sys20); # Copy fluent, gambit and fieldview status files from server
    system(sys21); # Copy ref from server
    system(sys22); # Copy res from server
    system(sys23); # Copy u^3 integral from server
    system(sys24); # Copy streamdata from server
  endif

  
  if (exist('res','file'))

    res = dlmread('res',',',1,0); #Uses octave-forge io function
    ref = dlmread('ref',',',1,0);
    ptot=max(ref(:,6)); # find upstream total pressure

    clear moddata;                     % Clears previously modified data
    moddata(:,1)=(res(:,2)*1000);      % Axial dimenisions in mm
    moddata(:,2)=(res(:,3)*1000);      % Tangential dimenisions in mm
    moddata(:,3)=(res(:,4)*1000);      % Radial dimenisions in mm
    moddata(:,4)=0;                    % In all cases turret=0
    # Note that if the axial vely = zero we might get NaN
    # Note also the element by element operator
    moddata(:,5)=atan(res(:,9)./res(:,7))*(180/pi()); # pitch angle
    moddata(:,6)=atan(res(:,8)./res(:,7))*(180/pi()); # yaw angle
    moddata(:,7)=(res(:,6))-ptot;
    moddata(:,8)=(res(:,5))-ptot;
    moddata=sortrows(moddata,[3 2]);

    # Fire the data out to a file....  
    fid = fopen("result.txt", 'w');
    fprintf(fid,"#axial \ttang \tradial \tturret \tpitch \tyaw \tptot \tpstat\n");
    
    for i=1:1:length(moddata) 
      if (! isnan(moddata(i,5)) || ! isnan(moddata(i,6))) # catch NaNs
	      fprintf(fid,"%5.5f\t", moddata(i,1));
	      fprintf(fid,"%5.5f\t", moddata(i,2));
	      fprintf(fid,"%5.5f\t", moddata(i,3));
	      fprintf(fid,"%5.5f\t", moddata(i,4));
	      fprintf(fid,"%5.5f\t", moddata(i,5));
	      fprintf(fid,"%5.5f\t", moddata(i,6));
        fprintf(fid,"%5.5f\t", moddata(i,7));
      	fprintf(fid,"%5.5f\n", moddata(i,8));
      endif
    endfor
    fclose(fid);

    system("oegularise result.txt result.t5c");
    system("field --data result --cfd --midspan=185");
    system("pitch --data result --tmin=-231 --tmax=-43 --rmin=0 --rmax=187.5");
    # system("newformat.py result.fld result.fldrr result.avg");

    # Need the yaw and SKE from the results file
    # Yaw angle is on line 11 and SKE is on line 13....
    # This code is bit rubbish..
    fid = fopen ("result.are");
    line_n = 0;
    do     
      txt = fgetl (fid);
      if (line_n == 10)
	      tmp= split(txt,"\t",2);
	      yaw = str2num(tmp(2,:));
      endif
      if (line_n == 12)
	      tmp= split(txt,"\t",2);
	      cske = str2num(tmp(2,:));
      endif
      line_n = line_n +1;
    until (txt == -1)
    fclose (fid);

    ## Process U^3 Integral
    fid = fopen("ucubed.srp");
    line_n = 0;
    do  
      txt = fgetl (fid);   
      if (line_n == 8)
        tmp = split(txt," ");
        ucubed = str2num(tmp(size(tmp,1),:));
      endif
    line_n = line_n +1;
    until (txt == -1)
    fclose (fid);

    ## Process Streamline Data
    %stream = streamprocess; % call streamline processing function    
    stream = 0; % DELETE THIS LINE WHEN YOU UNCOMMENT L139    

    ## Calculate Cost
    cost(chrom,1) = costfunction(yaw,cske,ucubed,stream);
    
  else # No file apply large cost penalty....
    cost(chrom,1) = 1000;
  endif
    
endfor

## Tidy up files on the server to save space
sys25 = sprintf("ssh %s@vega3.dur.ac.uk 'rm -rf %sGeneration_%d'"
   		,user_id,cluster_dir,iga); 
system(sys25); # real men use rm -rf in scripts...

# Generate updated system log
for a=1:1:popsize
  par(a,(npar+1))=cost(a,1);
  log((((popsize*(iga)))+a),(npar+1))=cost(a,1);
end

chdir(sprintf("%s",base_dir)); % Return to base directory

endfunction
